# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Terminal app autopilot tests."""

import sqlite3
import os.path

import fixtures
from autopilot.input import Mouse, Touch, Pointer
from autopilot.platform import model
from autopilot.testcase import AutopilotTestCase
from ubuntuuitoolkit import emulators as toolkit_emulators

from ubuntu_terminal_app import emulators


class TerminalTestCase(AutopilotTestCase):

    """A common testcase class that provides useful methods for the terminal
    app.

    """
    if model() == 'Desktop':
        scenarios = [('with mouse', dict(input_device_class=Mouse))]
    else:
        scenarios = [('with touch', dict(input_device_class=Touch))]

    def setUp(self):
        self.pointing_device = Pointer(self.input_device_class.create())
        super(TerminalTestCase, self).setUp()

        self.EXEC = 'terminal'
        self.source_dir = os.path.dirname(
            os.path.dirname(os.path.abspath('.')))
        self.build_dir = self._get_build_dir()
        self.local_location_binary = os.path.join(self.build_dir,
                                                  'src', 'app', self.EXEC)
        self.installed_location_binary = os.path.join('/usr/bin/', self.EXEC)
        self.installed_location_qml = \
            '/usr/share/terminal/qml/ubuntu-terminal-app.qml'

        if os.path.exists(self.local_location_binary):
            self.launch_test_local()
        elif os.path.exists(self.installed_location_binary):
            self.launch_test_installed()
        else:
            self.launch_test_click()

    def _get_build_dir(self):
        build_dir = self.source_dir

        return build_dir

    def launch_test_local(self):
        self.useFixture(fixtures.EnvironmentVariable(
            'QML2_IMPORT_PATH', newvalue=os.path.join(self.build_dir,
                                                      'src', 'plugin')))

        self.app = self.launch_test_application(
            self.local_location_binary,
            app_type='qt',
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    def launch_test_installed(self):
        self.app = self.launch_test_application(
            self.installed_location_binary,
            '-q', self.installed_location_qml,
            app_type='qt',
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    def launch_test_click(self):
        self.app = self.launch_click_package(
            "com.ubuntu.terminal",
            emulator_base=toolkit_emulators.UbuntuUIToolkitEmulatorBase)

    @property
    def main_view(self):
        return self.app.select_single(emulators.MainView)


class DbMan(object):

    """
    Helper functions for dealing with sqlite databases
    """

    def get_db(self):
        dbs_path = os.path.expanduser(
            "~/.local/share/com.ubuntu.terminal/Databases")
        if not os.path.exists(dbs_path):
            return None

        files = [f for f in os.listdir(dbs_path)
                 if os.path.splitext(f)[1] == ".ini"]
        for f in files:
            ini_path = os.path.join(dbs_path, f)
            with open(ini_path) as ini:
                for line in ini:
                    if "=" in line:
                        key, val = line.strip().split("=")
                        if key == "Name" and val == "UbuntuTerminalDB":
                            try:
                                return ini_path.replace(".ini", ".sqlite")
                            except OSError:
                                pass
        return None

    def clean_db(self):
        path = self.get_db()
        if path is None:
            self.assertNotEquals(path, None)
        try:
            os.remove(path)
        except OSError:
            pass

    def get_font_size_from_storage(self):
        db = self.get_db()
        conn = sqlite3.connect(db)

        with conn:
            cur = conn.cursor()
            cur.execute("SELECT value FROM config WHERE item='fontSize'")
            row = cur.fetchone()
            if row is None:
                self.assertNotEquals(row, None)
            return int(row[0])

    def get_color_scheme_from_storage(self):
        db = self.get_db()
        conn = sqlite3.connect(db)

        with conn:
            cur = conn.cursor()
            cur.execute("SELECT value FROM config WHERE item='colorScheme'")
            row = cur.fetchone()
            if row is None:
                self.assertNotEquals(row, None)
            return row[0]
