# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Terminal app autopilot tests."""

from __future__ import absolute_import

from autopilot.matchers import Eventually
from testtools.matchers import Equals, Not, Is
from autopilot.platform import model

from ubuntu_terminal_app.tests import TerminalTestCase, DbMan

from time import sleep
import unittest
import random


class TestMainWindow(TerminalTestCase, DbMan):

    def setUp(self):
        super(TestMainWindow, self).setUp()
        self.assertThat(
            self.main_view.visible, Eventually(Equals(True)))

    def tearDown(self):
        super(TestMainWindow, self).tearDown()

    def hide_panels(self):
        """Click hide panels button"""
        toolbar = self.main_view.open_toolbar()
        timeout = 0
        toolbar.click_button("PanelsButton")
        panel_popover = self.main_view.get_panel_actions_popover()

        # we have to poll the complex action because
        # we can't check the animation for the header
        while panel_popover is None and timeout < 10:
            toolbar.click_button("PanelsButton")
            panel_popover = self.main_view.get_panel_actions_popover()
            sleep(1)
            timeout += 1
        panel_popover.click_button("Hide extra panel")

    def click_item_selector_item(self, selector, value):
        """Clicks item from item selector"""
        # The item selector is assumed to be the following format
        # ListItem.ItemSelector {
        #    objectName:
        #    model:
        self.assertThat(lambda: self.main_view.select_single("ItemSelector",
                        objectName=selector), Eventually(Not(Is(None))))
        item_selector = self.main_view.select_single("ItemSelector",
                                                     objectName=selector)
        # we need to regrab object as for some reason it can sometimes
        # change and our ordering will be off down below
        self.assertThat(lambda: self.main_view.select_single("ItemSelector",
                        objectName=selector), Eventually(Not(Is(None))))
        item_selector = self.main_view.select_single("ItemSelector",
                                                     objectName=selector)
        self.assertThat(lambda: item_selector.select_many("QQuickColumn"),
                        Eventually(Not(Is(None))))
        rows = item_selector.select_many("QQuickColumn")
        self.assertThat(lambda: rows[0].select_single("QQuickListView",
                        objectName="listView"), Eventually(Not(Is(None))))
        items = rows[0].select_single("QQuickListView", objectName="listView")
        self.pointing_device.click_object(items)
        for ritem in items:
            if ritem.text == value:
                self.pointing_device.click_object(ritem)
                # the first label is always the "color scheme"
                itemLabel = lambda: ritem.text
                self.assertThat(itemLabel, Eventually(Equals(value)))

    def test_control_panel(self):
        """Make sure that Control Keys Panel is visible
        when clicking the toolbar popup items and hides properly."""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button("PanelsButton")

        panel_popover = self.main_view.get_panel_actions_popover()
        panel_popover.click_button("Control keys")

        panelCtrl = lambda: self.main_view.get_control_panel().visible
        self.assertThat(panelCtrl, Eventually(Equals(True)))
        self.hide_panels()
        self.assertThat(panelCtrl, Eventually(Equals(False)))

    def test_function_panel(self):
        """Make sure that function Keys Panel is visible
        when clicking the toolbar popup items and hides properly."""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button("PanelsButton")

        panel_popover = self.main_view.get_panel_actions_popover()
        panel_popover.click_button("Function keys")

        panelFunc = lambda: self.main_view.get_function_panel().visible
        self.assertThat(panelFunc, Eventually(Equals(True)))
        self.hide_panels()
        self.assertThat(panelFunc, Eventually(Equals(False)))

    def test_textctrl_panel(self):
        """Make sure that Text Control Keys Panel is visible
        when clicking the toolbar popup items and hides properly."""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button("PanelsButton")

        panel_popover = self.main_view.get_panel_actions_popover()
        panel_popover.click_button("Text ctrl keys")

        panelScrl = lambda: self.main_view.get_scroll_panel().visible
        self.assertThat(panelScrl, Eventually(Equals(True)))
        self.hide_panels()
        self.assertThat(panelScrl, Eventually(Equals(False)))

    def test_circle_menu_shows(self):
        """Make sure that Circle Menu is visible
        on long tap"""
        kterm = self.main_view.get_terminal_page()
        x, y, w, h = kterm.globalRect
        tap_x = (x + w) / 2
        tap_y = (y + h) / 3

        # tap in the top third of the screen, to avoid OSK
        self.pointing_device.move(tap_x, tap_y)
        self.pointing_device.press()
        # we can hold the press for a long time without issue
        # so we'll ensure the app recieves our signal when under load
        sleep(4)
        self.pointing_device.release()

        menu = self.main_view.get_circle_menu()
        self.assertThat(menu.visible, Eventually(Equals(True)))

    def test_header(self):
        """Make sure that Header is visible
        in Portrait Mode and not visible in landscape mode"""
        header = lambda: self.main_view.get_header().visible
        if model() == "Nexus 10" or model() == "Nexus 7":
            self.assertThat(header, Eventually(Equals(False)))
        else:
            self.assertThat(header, Eventually(Equals(True)))

    @unittest.skip("This test randomly fails and is not consistent")
    def test_color_scheme_changes(self):
        """Make sure that color scheme is set correctly"""
        kterm = self.main_view.get_kterm()
        self.main_view.switch_to_tab("SettingsTab")

        # are these string translatable?
        # if so, we need to do this another way
        schemeList = ("DarkPastels",
                      "Linux",
                      "BlackOnRandomLight",
                      "BlackOnWhite",
                      "BlackOnLightYellow",
                      "GreenOnBlack",
                      "WhiteOnBlack")

        for scheme in schemeList:
            self.click_item_selector_item("liSchemes", scheme)
            # This assert uses KTerminalDisplay property (old rev)
            self.assertThat(kterm.colorScheme, Eventually(Equals(scheme)))
            # This assert uses localstorage (new rev)
            colorScheme = self.get_color_scheme_from_storage
            self.assertThat(colorScheme, Eventually(Equals(scheme)))

    def test_font_size_changes(self):
        """Make sure that font size is set correctly"""
        toolbar = self.main_view.open_toolbar()
        toolbar.click_button("SettingsButton")

        # change font size to max
        self.main_view.drag_horizontal_slider("slFont", 32)
        sleep(1)
        font_size = self.get_font_size_from_storage()
        self.assertThat(font_size, Equals(32))

        # change font size to min
        self.main_view.drag_horizontal_slider("slFont", 8)
        sleep(1)
        font_size = self.get_font_size_from_storage()
        self.assertThat(font_size, Equals(8))

        # change font size to random sizes
        for loop in range(1, 3):
            randSize = random.randrange(8, 32, 1)
            self.main_view.drag_horizontal_slider("slFont", randSize)
            sleep(1)
            font_size = self.get_font_size_from_storage()
            self.assertThat(font_size, Equals(randSize))
